

from abc import ABC, abstractmethod
from selenium import webdriver

from stores.data_store_interface import IDataStore
from drivers.driver_creator import DriverCreator
from utils.logger_helper import LoggerHelper
from utils.config_helper import ConfigHelper
from models.collect_data import CollectData
from models.process_data import ProcessData


class IDataCollectionAdapter(ABC):
    """
    数据收集适配器抽象类
    """
    _url = ""
    _store = None
    _driver = None
    _keyword = None
    _adapter_type = ""

    logger = LoggerHelper.get_logger()
    config = ConfigHelper()


    @property
    def search_day_key(self) -> str:
        return f"adapter.{self._adapter_type}.search_day"
    @property
    def batch_save_key(self) -> str:
        return f"adapter.{self._adapter_type}.batch_save"
    @property
    def store(self) -> IDataStore:
        return self._store

    @property
    def url(self):
        return self._url

    @property
    def keyword(self):
        return self._keyword

    @property
    def driver(self) -> webdriver:
        if not self._driver:
            self._driver = self._create_driver()
        return self._driver

    def _create_driver(self) -> webdriver:
        try:
            return DriverCreator().gen_remote_driver(self.url)
            # return DriverCreator().gen_chrome_driver(self.url)
        except Exception as e:
            raise Exception(f"创建驱动器失败: {e}")

    @abstractmethod
    def login(self, username: str, password: str) -> None:
        """
        如果需要登录，则登录后跳转到搜索页面（不自动跳转的需要手动执行）

        :param username: 用户名
        :type username: str
        :param password: 密码
        :type password: str
        :raises Exception: 如果登录失败，应抛出异常
        """
        try:
            # 实现登录逻辑
            pass
        except Exception as e:
            raise Exception(f"登录失败: {e}")

    @abstractmethod
    def _search(self, keyword: str) -> list:
        """
        根据关键字搜索，返回搜索结果列表

        :param keyword: 搜索关键字
        :type keyword: str
        :return: 搜索结果列表
        :rtype: list
        :raises Exception: 如果搜索失败，应抛出异常
        """
        try:
            results = []
            # 实现搜索逻辑
            return results if results else []
        except Exception as e:
            raise Exception(f"搜索失败: {e}")

    @abstractmethod
    def collect(self, keyword: str, store: IDataStore) -> None:
        """
        处理搜索结果列表，返回处理后的数据列表

        :param keyword: 搜索结果列表
        :param store: 数据储存库
        :type keyword: str
        :return: 处理后的数据列表
        :rtype: list
        :raises Exception: 如果处理失败，应抛出异常
        """
        try:
            if keyword:
                # 实现处理逻辑
                pass
        except Exception as e:
            raise Exception(f"处理失败: {e}")

    def teardown(self) -> None:
        """
        关闭浏览器驱动器

        :raises Exception: 如果关闭驱动器失败，应抛出异常
        """
        try:
            if self.driver:
                self.driver.quit()
        except Exception as e:
            raise Exception(f"关闭驱动器失败: {e}")

    def _check_is_collect_by_url(self, url: str) -> bool:
        old = self.store.query_one_collect_url(url)
        if old:
            self.logger.info(f"已采集过: {url}")
            return True
        return False
    def _check_content(self,content) -> bool:
        collect_data_key = self.config.get("save.collect_data_key")
        if not collect_data_key:
            self.logger.info("未配置 save.collect_data_key，跳过内容检查")
            return True
        # self.logger.info(f"检查数据有效性: {collect_data_key}")
        collect_data_key = collect_data_key.replace("，", ",")
        keys = collect_data_key.split(",")
        keys = [key.strip() for key in keys]
        for key in keys:
            key = key.strip()
            # self.logger.info(f"检查数据有效性: {key}")
            if key in content:
                self.logger.info(f"有效数据: {self.driver.current_url}")
                return True

        return False



    def _save_db(self, url, content, attach_str = None,is_invalid=False):
        if not self.store:
            self.logger.info(f"DataStore 未指定: {url}，关键字{self.keyword}")
            return False
        else:
            status = 2 if is_invalid else 0
            data = CollectData(url, self.keyword, content, attach_str, status)
            self.store.insert_collect_data(data, self.config.get_bool(self.batch_save_key))
            return True

