from utils.logger_helper import LoggerHelper
from utils.email_helper import EmailHelper
from stores.data_store_interface import IDataStore
from models.process_data import ProcessData


class DataSend:
    logger = LoggerHelper.get_logger()
    _error_arr = []

    @property
    def store(self) -> IDataStore:
        return self._store

    def __init__(self, store: IDataStore):
        self._store = store

    def send(self) -> None:
        self._error_arr = []
        list = self.store.query_to_send()
        self.logger.info(f"开始发送邮件，数量为 {len(list)}")
        for item in list:
            self._send_item(item)
        if len(self._error_arr) > 0:
            self._send_email_no_found()

    def _send_item(self, item: ProcessData) -> None:
        self.logger.info(f"开始发送邮件，地区为：{item.area} ，URL为 {item.url}")
        email = self.store.get_email_by_area(item.area)
        if not email:
            self.logger.error(f"{item.area} 下没有找到email")
            if item.area not in self._error_arr:
                self._error_arr.append(item.area)
            return
        body = self._build_email_content(item)
        flag = EmailHelper().send_email(email, item.title, body, True, item.attach_path)
        if flag:
            self.store.set_send(item.no)

    @staticmethod
    def _build_email_content(item: ProcessData, other: str = "") -> str:
        html_body = f"""
        <html>
        <head>
            <style>
                body {{
                    background-color: #f4f4f9;
                    font-family: Arial, sans-serif;
                    margin: 0;
                    padding: 20px;
                }}
                h1 {{
                    text-align: center;
                    color: #333;
                }}
                .container {{
                    max-width: 600px;
                    margin: 0 auto;
                    background-color: #fff;
                    padding: 20px;
                    border-radius: 8px;
                    box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
                }}
                .button-container {{
                    text-align: center;
                    margin-top: 20px;
                }}
                .button {{
                    display: inline-block;
                    padding: 10px 20px;
                    font-size: 16px;
                    color: #fff!important;
                    background-color: #007bff;
                    text-decoration: none;
                    border-radius: 5px;
                    transition: background-color 0.3s;
                }}
                .button:hover {{
                    background-color: #0056b3;
                }}
                .system {{
                    color: #aaa;
                }}

            </style>
        </head>
        <body>
            <div class="container">
                <h1>{item.title}</h1>
                <p><strong>搜索关键字:</strong> {item.keyword}</p>
                <p><strong>发布日期:</strong> {item.release_date}</p>
                <p><strong>招标编号:</strong> {item.no}</p>
                <p><strong>开标时间:</strong> {item.date}</p>
                <p><strong>开标地点:</strong> {item.address}</p>
                <p><strong>标书摘要:</strong> {item.summary}</p>
                <div class="button-container">
                    <a href="{item.url}" class="button">查看详情</a>
                </div>
                <div>
                    <h3>{other}</h3>
                </div>
                <p class="system">本邮件由系统自动发送，请勿回复。</p>

            </div>
        </body>
        </html>
        """
        return html_body

    def _send_email_no_found(self) -> None:
        email = EmailHelper().config.get("email.error_email")
        self.logger.info(f"开始发送区域邮箱未匹配邮件: {email}")
        if not email:
            return
        title = "Warning: 相关地区没有匹配到邮箱，请及时添加相关配置"
        content = "以下区域中没有配置邮箱：\n\n    "
        content += "、".join(self._error_arr)
        content += "\n\n请及时添加相关配置。"
        EmailHelper().send_email(email, title, content, False, None)
