from dateutil import parser
import schedule

from utils.logger_helper import LoggerHelper
from utils.config_helper import ConfigHelper
from stores.mysql_data_store import MysqlDataStore
from models.url_setting import UrlSetting
from main.data_collector import DataCollector
from main.data_process import DataProcess
from main.data_send import DataSend
from utils.email_helper import EmailHelper
from utils.file_helper import FileHelper

class Runner:
    logger = LoggerHelper.get_logger()
    config = ConfigHelper()
    store = MysqlDataStore()  # 复用 store 对象

    def run(self):
        self.logger.info("应用程序已启动！")
        urls = UrlSetting().fetch_all()
        if not urls or len(urls) == 0:
            self.logger.error("未找到任何 URL 设置")
            return
        self.logger.info(f"共找到 {len(urls)} 个 URL 设置")

        collect_time = self.config.get("schedule.collect")
        process_time = self.config.get("schedule.process")
        send_email_time = self.config.get("schedule.send_email")

        collect_times = self._validate_and_format_time(collect_time, ["06:00"])
        for time in collect_times:
            self.logger.info(f"{time} 执行 采集处理数据 任务")
            schedule.every().day.at(time).do(self._collect_process_job)

        process_times = self._validate_and_format_time(
            process_time, ["10:00", "15:00", "19:00"])
        for time in process_times:
            self.logger.info(f"{time} 执行  AI处理数据  任务")
            schedule.every().day.at(time).do(self._process_job)

        send_email_times = self._validate_and_format_time(
            send_email_time, ["08:20", "14:00"])
        for time in send_email_times:
            self.logger.info(f"{time} 执行   发送邮件   任务")
            schedule.every().day.at(time).do(self._send_job)
        if self.config.get_bool("schedule.run_now"):
            self.logger.info("立即执行任务")
            self._collect_process_job()
            # self._send_job()
            # self._process_job()

    def _collect_process_job(self):
        try:
            self.logger.info("开始执行数据采集处理任务")
            url_setting = UrlSetting()
            for url_setting in url_setting.fetch_all():
                data_collector =None
                try:
                    self.logger.info(f"开始采集: {url_setting.url}")
                    data_collector = DataCollector(url_setting.adapter_type,
                                                   url_setting.url,
                                                   url_setting.username,
                                                   url_setting.password,
                                                   self.store)
                    keywords = url_setting.keywords
                    keyword_array = keywords.split(',')
                    for keyword in keyword_array:
                        data_collector.collect(keyword)
                    self.logger.info(f"采集完成: {url_setting.url}")
                except Exception as e:
                    self._send_error_email(
                        "数据采集",
                        f"\n    Type: {url_setting.adapter_type} \n    Url: {url_setting.url}\n    错误: {str(e)}"
                    )
                    self.logger.error(f"采集发生异常: {e}")
                finally:
                    if data_collector:
                        data_collector.close()

                try:
                    self.logger.info(f"开始AI处理: {url_setting.url}")
                    data_process = DataProcess(self.store)
                    data_process.process()
                except Exception as e:
                    self._send_error_email(
                        "AI数据处理",
                        f"\n    Type: {url_setting.adapter_type} \n    Url: {url_setting.url}\n    错误: {str(e)}"
                    )
                    self.logger.error(f"AI处理发生异常: {e}")
                    break  # 中断当前 URL 设置的处理
            self.logger.info("数据采集处理任务执行完毕")
        except Exception as e:
            self.logger.error(f"数据采集处理任务执行失败: {e}")

    def _process_job(self):
        try:
            self.logger.info("开始AI处理数据执行任务")
            data_process = DataProcess(self.store)
            data_process.process()
            self.logger.info("AI处理数据任务执行完毕")
        except Exception as e:
            self._send_error_email("AI数据处理", f"\n    错误: {str(e)}")
            self.logger.error(f"AI任务执行失败: {e}")

    def _send_job(self):
        try:
            self.logger.info("开始邮件发送执行任务")
            DataSend(self.store).send()
            self.logger.info("邮件发送任务执行完毕")
        except Exception as e:
            self._send_error_email("邮件发送", f"\n    错误: {str(e)}")
            self.logger.error(f"邮件发送任务执行失败: {e}")

    def _validate_and_format_time(self, time_str, default_time: list):
        """验证并格式化时间字符串"""
        if not time_str:
            return default_time
        time_str = time_str.strip().replace('，', ',')
        # 分割字符串为列表
        items = [
            item.strip().strip("'").strip('"') for item in time_str.split(',')
        ]

        # 初始化结果列表
        formatted_times = []

        for item in items:
            if not item:
                continue  # 跳过空字符串
            try:
                item = item.replace('：', ':')
                # 使用 dateutil.parser 解析时间字符串
                parsed_time = parser.parse(item).time().strftime('%H:%M:%S')
                formatted_times.append(parsed_time)
            except Exception as e:
                self.logger.error(f"配置时间解析错误: {item},: {e} ")
        if len(formatted_times) == 0:
            self.logger.error(f"解析时间失败，使用默认时间 {default_time}")
            return default_time
        return formatted_times

    def _send_error_email(self, title: str, error: str) -> None:
        email_helper = EmailHelper()
        email = self.config.get("email.error_email")
        self.logger.info(f"发送错误邮件: {email}")
        if not email:
            return
        title = f"{title}异常"
        content = f"{title}，请及时处理。\n\n异常信息：{error}"
        email_helper.send_email(email, title, content, False, None)
