from datetime import datetime
from utils.mysql_helper import MySQLHelper
from utils.logger_helper import LoggerHelper


class CollectData:

    logger = LoggerHelper.get_logger()
    UNPROCESSED = 0
    PROCESSED = 1
    INVALID = 2

    def __init__(self,
                 url=None,
                 keyword=None,
                 content=None,
                 attach_path=None,
                 status=UNPROCESSED,
                 create_time=None,
                 process_time=None):
        self.url = url
        self.keyword = keyword
        self.content = content
        self.attach_path = attach_path
        self.status = status
        self.create_time = create_time or datetime.now()
        self.process_time = process_time

    def __repr__(self):
        return (
            f"CollectData(url={self.url}, keyword={self.keyword}, "
            f"content={self.content}, status={self.status}, "
            f"create_time={self.create_time}, process_time={self.process_time})"
        )

    _insert_query = """
        INSERT IGNORE INTO t_collect_data (url, keyword, content, attach_path, status, create_time)
        VALUES (%s, %s, %s, %s, %s, %s);
        """
    _insert_query_history = """
         INSERT IGNORE INTO t_collect_data_history (url, keyword, content, attach_path, status, create_time)
         VALUES (%s, %s, %s, %s, %s, %s);
         """
    _delete_query = """
         DELETE FROM t_collect_data
         WHERE url = %s;
         """
    def insert(self, collect_data):
        if not isinstance(collect_data, self.__class__):
            raise TypeError("collect_data 不是 CollectData 的实例")
        with MySQLHelper() as db_helper:

            params = (collect_data.url, collect_data.keyword,
                       collect_data.content,collect_data.attach_path,
                      collect_data.status, datetime.now())
            if collect_data.status == self.INVALID:
                db_helper.execute_non_query(self._insert_query_history, params)
            else:
                db_helper.execute_non_query(self._insert_query, params)

    def insert_batch(self, collect_data_list):
        if not all(
                isinstance(collect_data, self.__class__)
                for collect_data in collect_data_list):
            raise TypeError("collect_data_list 中的所有元素必须是 CollectData 的实例")

        params = [
            (
                collect_data.url,
                collect_data.keyword,
                collect_data.content,
                collect_data.attach_path,
                collect_data.status,
                datetime.now()  # 每次调用 datetime.now() 获取当前时间
            ) for collect_data in collect_data_list
            if collect_data.status != 2
        ]
        params2 = [
            (
                collect_data.url,
                collect_data.keyword,
                collect_data.content,
                collect_data.attach_path,
                collect_data.status,
                datetime.now()  # 每次调用 datetime.now() 获取当前时间
            ) for collect_data in collect_data_list
            if collect_data.status == 2
        ]

        with MySQLHelper() as db_helper:
            db_helper.execute_non_query(self._insert_query, params)
            # 获取受影响的行数
            affected_rows = db_helper.connection.affected_rows()

            db_helper.execute_non_query(self._insert_query_history, params2)
            self.logger.info(f"成功插入 {affected_rows} 条数据")
            return affected_rows

    # def insert_url(self, url: str, keyword: str, content: str):
    #     with MySQLHelper() as db_helper:
    #         query = """
    #             INSERT IGNORE INTO t_collect_data (url, keyword, content, status, create_time, process_time)
    #             VALUES (%s, %s, %s, %s, %s, %s)
    #             """
    #         params = (url, keyword, content, 0, datetime.now, None)
    #         db_helper.execute_non_query(query, params)

    # def fetch_all():
    #     with MySQLHelper() as db_helper:
    #         query = "SELECT * FROM t_collect_data"
    #         results = db_helper.execute_query(query)
    #         data = [CollectData(**result) for result in results]
    #         return data

    def fetch_all_urls(self) -> list[str]:
        with MySQLHelper() as db_helper:
            query = "SELECT url FROM t_collect_data"
            results = db_helper.execute_query(query)
            # 使用列表推导式一次性提取所有 'url' 值
            data = [result['url'] for result in results]
            return data

    def fetch_urls_to_process(self) -> list[str]:
        with MySQLHelper() as db_helper:
            query = """
            SELECT url
            FROM t_collect_data
            WHERE status = 0
            """
            results = db_helper.execute_query(query)
            data = [result['url'] for result in results]
            return data

    def fetch_one_url(self, url: str):
        with MySQLHelper() as db_helper:
            query = """
             SELECT url FROM `t_collect_data_history` WHERE url= %s UNION SELECT url FROM `t_collect_data`  WHERE url= %s LIMIT 1
            """
            result = db_helper.fetch_one(query, (url, url))
            if not result:
                return None
            data = result["url"]
            return data

    def fetch_one_collect_by_url(self, url: str):
        with MySQLHelper() as db_helper:
            query = """
                SELECT url,keyword,content,status FROM t_collect_data WHERE url = %s  LIMIT 1
            """
            result = db_helper.fetch_one(query, (url, ))
            if not result:
                return None
            data = CollectData(url=result["url"],
                               keyword=result["keyword"],
                               content=result["content"],
                               status=result["status"])
            return data

    def set_process(self, url: str):
        # with MySQLHelper() as db_helper:
        #     query = """
        #     UPDATE t_collect_data
        #     SET status = 1
        #     WHERE url = %s
        #     """
        #     db_helper.execute_non_query(query, (url))
        self.move_to_history_and_delete(url)


    def move_to_history_and_delete(self, url: str):
        with MySQLHelper() as db_helper:
            # 查询 t_collect_data 中的数据
            query = """
             SELECT url, keyword, content, attach_path, status, create_time, process_time
             FROM t_collect_data
             WHERE url = %s
             """
            result = db_helper.fetch_one(query, (url,))
            if not result:
                self.logger.warning(f"URL {url} 未在 t_collect_data 中找到，无法移动到历史表并删除。")
                return False

            # 将数据插入到 t_collect_data_history
            insert_query = self._insert_query_history
            insert_params = (
                result["url"],
                result["keyword"],
                result["content"],
                result["attach_path"],
                result["status"],
                result["create_time"]
            )
            db_helper.execute_non_query(insert_query, insert_params)

            # 删除 t_collect_data 中的数据
            delete_query = self._delete_query
            delete_params = (url,)
            db_helper.execute_non_query(delete_query, delete_params)

            self.logger.info(f"URL {url} 已从 t_collect_data 移动到 t_collect_data_history 并删除。")
            return True

    def fetch_by_status(self, status=0):
        with MySQLHelper() as db_helper:
            query = """
            SELECT url, keyword, content, status, create_time, process_time
            FROM t_collect_data
            WHERE status = %s
            """
            results = db_helper.execute_query(query, (status, ))
            data = [CollectData(**result) for result in results]
            return data

    def set_status(self, collect_data):
        if not isinstance(collect_data, self):
            raise TypeError("collect_data 不是 CollectData 的实例")
        with MySQLHelper() as db_helper:
            query = """
            UPDATE t_collect_data
            SET status = %s
            WHERE url = %s
            """
            params = (collect_data.status, collect_data.url)
            db_helper.execute_non_query(query, params)
