import re
import requests
from openai import OpenAI
import json

from utils.logger_helper import LoggerHelper
from utils.config_helper import ConfigHelper
from models.process_data import ProcessData


class AiHelper:

    logger = LoggerHelper.get_logger()
    config = ConfigHelper()

    _ai_api_key = None
    _ai_api_url = None
    _ai_max_tokens = 150
    DEFAULT_AI_SYSTEM_PROMPT = "请帮我分析以下文字，提取出关键信息，并以json格式字符串返回，如果部分信息为空，则该字段返回为空。"
    DEFAULT_AI_PROMPT_TEMPLATE = """在以上内容中提取信息：编号(no) 、标题(title)、在哪个城市招标(area)、开标的时间(date)、
    开标的地点(address)、发布时间(release_date)、150字左右的招标条件要求及联系方式等内容摘要(summary),相关采购设备的名称信息,多个设备以逗号分割(device)。
    返回包含no,title,area,date,address,release_date,summary,device字段的json格式字符串，没有找到或未提供的信息json字段为空。
"""

    def __init__(self):
        self._ai_api_key = self.config.get("ai.key")
        self._ai_api_url = self.config.get("ai.url")
        self._api_model = self.config.get("ai.model")
        max_tokens = self.config.get("ai.max_tokens")
        if max_tokens:
            self._ai_max_tokens = int(max_tokens)
        self._ai_system_prompt = self.config.get("ai.system_prompt",
                                                 self.DEFAULT_AI_SYSTEM_PROMPT)
        self._ai_prompt_template = self.config.get(
            "ai.prompt_template", self.DEFAULT_AI_PROMPT_TEMPLATE)


    def call_ai(self, content: str) -> ProcessData:
        # 截取前100个字符进行日志记录
        # truncated_content = content[:100]
        self.logger.info("调用AI API")
        if self._ai_api_key is None:
            raise Exception("AI API key 没有配置")
        if self._ai_api_url is None:
            raise Exception("AI API url 没有配置")
        if self._api_model is None:
            raise Exception("AI API model 没有配置")
        client = OpenAI(api_key=self._ai_api_key, base_url=self._ai_api_url)
        completion = client.chat.completions.create(
            model=self._api_model,
            messages=[{
                "role": "system",
                "content": self._ai_system_prompt,
            }, {
                "role": "user",
                "content": f"{content}  {self._ai_prompt_template}",
            }],
            stream=False,
            temperature=0.7,
        )

        self.logger.info(f"AI Response: {completion.model_dump_json()}")
        response = json.loads(completion.model_dump_json())
        #self.logger.info(f"AI Response: {response}")
        try:
            res_str = self._extract_message_content(response)
            return self._parse_response(res_str, True)
        except Exception as e:
            raise Exception(f"解析 AI 响应错误: {e}")

    @staticmethod
    def _extract_message_content(response_json: dict) -> str:
        if "choices" in response_json and len(response_json["choices"]) > 0:
            choice = response_json["choices"][0]
            message_content = choice.get("message", {}).get("content", "")
        elif "message" in response_json:
            message_content = response_json["message"].get("content", "")
        else:
            raise Exception("AI 响应中未找到有效的 choices 或 message 数据")

        # 移除多余的 ```json 和 ```
        if message_content.startswith("```json") and message_content.endswith(
                "```"):
            message_content = message_content[6:-3]

        # 去除开头的 'n' 字符
        if message_content.startswith('n'):
            message_content = message_content[1:]
        # 移除无效的转义字符和时间戳前缀
        message_content = re.sub(r'\\[0-9]{2}', '',
                                 message_content)  # 移除 \32 等无效转义字符
        message_content = re.sub(r'\d{4}-\d{2}-\dT\d{2}:\d{2}:\d{2}\.\d+Z', '',
                                 message_content)  # 移除时间戳
        message_content = message_content.strip()  # 去除首尾空白字符

        # 替换所有的反斜杠
        message_content = message_content.replace("\\", "")

        return message_content

    def _parse_response(self, response: str, first=True) -> ProcessData:
        self.logger.info(f"AI Response JSON STR: {response}")
        try:
            data = json.loads(response)
            return ProcessData(no=data.get("no"),
                               title=data.get("title"),
                               date=data.get("date"),
                               area=data.get("area"),
                               address=data.get("address"),
                               devices=data.get("device"),
                               summary=data.get("summary"),
                               release_date=data.get("release_date"))
        except json.JSONDecodeError as e:
            if first:
                self.logger.error(f"JSON 解析错误,去除部分特殊字符重新解析一次: {e}")
                # 替换中文引号为空
                message_content = re.sub(r'[“”]', "", response)  # 替换双引号
                message_content = re.sub(r'[‘’]', "", message_content)  # 替换单引号
                return self._parse_response(message_content, False)
            else:
                raise Exception(f"解析 AI 响应错误: {e}")


    def call_ai_1(self, content: str) -> ProcessData:
        # 截取前100个字符进行日志记录
        # truncated_content = content[:100]
        self.logger.info("调用AI API")
        if self._ai_api_key is None:
            raise Exception("AI API key 没有配置")
        if self._ai_api_url is None:
            raise Exception("AI API url 没有配置")
        if self._api_model is None:
            raise Exception("AI API model 没有配置")
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self._ai_api_key}"
        }
        messages = [{
            "role": "system",
            "content": self._ai_system_prompt
        }, {
            "role": "user",
            "content": f"{content} {self._ai_prompt_template}"
        }]

        data = {
            "model": self._api_model,
            "messages": messages,
            "stream": False,
            "max_tokens": self._ai_max_tokens
        }
        response = requests.post(self._ai_api_url, headers=headers, json=data)
        if response.status_code == 200:
            try:
                self.logger.info(f"AI Response: {response.text}")
                res_str = self._extract_message_content(response.json())
                return self._parse_response(res_str, True)
            except Exception as e:
                raise Exception(f"解析 AI 响应错误: {e}")
        else:
            raise Exception(
                f"调用 AI 错误: {response.status_code} - {response.text}")
