import os
import logging
from logging.handlers import TimedRotatingFileHandler

from utils.config_helper import ConfigHelper

class LoggerHelper:
    """
    日志辅助类，用于创建和提供日志记录器实例
    该类实现了单例模式，确保在整个应用程序中只有一个日志记录器实例被创建和使用
    """
    _instance = None
    config = ConfigHelper()

    def __new__(cls, *args, **kwargs):
        """
        实现单例模式，确保日志记录器仅被创建一次
        如果尚未创建实例，则创建并初始化日志记录器
        """
        if not cls._instance:
            cls._instance = super(LoggerHelper,
                                  cls).__new__(cls, *args, **kwargs)
            try:
                cls._instance._initialize_logger()
            except Exception as e:
                raise Exception(f"配置logger出错: {e}")
        return cls._instance

    @property
    def logger(self):
        return self._logger

    def _initialize_logger(self):
        """
        初始化日志记录器，包括设置日志级别、创建处理器和格式化器，并将它们组合起来
        """
        self._logger = logging.getLogger('app_logger')
        self._logger.setLevel(logging.INFO)
        log_file_path = self.config.get("logger.file_path", "./logs")
        if not os.path.exists(log_file_path):
            os.makedirs(log_file_path)

        # 创建按日期分割的文件处理器
        file_handler = TimedRotatingFileHandler(os.path.join(log_file_path, 'crawler.log'),
                                                when='midnight',
                                                interval=1,
                                                backupCount=7,
                                                encoding='utf-8')
        file_handler.setLevel(logging.INFO)

        # 创建控制台处理器
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)

        # 创建格式化器
        formatter = logging.Formatter(
            '%(asctime)s - %(levelname)s - %(message)s')

        # 将格式化器添加到处理器
        file_handler.setFormatter(formatter)
        console_handler.setFormatter(formatter)

        # 将处理器添加到日志记录器
        self._logger.addHandler(file_handler)
        self._logger.addHandler(console_handler)

    @classmethod
    def get_logger(cls):
        """
        提供初始化后的日志记录器实例
        :return: 初始化后的日志记录器实例
        """
        if not cls._instance:
            cls._instance = cls()
        return cls._instance._logger
